"""added device ip

Peek Plugin Database Migration Script

Revision ID: de24e18d0e6e
Revises: 97d44e5aeb18
Create Date: 2023-10-05 15:58:55.136116

"""

# revision identifiers, used by Alembic.
revision = "de24e18d0e6e"
down_revision = "97d44e5aeb18"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "DeviceInfo",
        sa.Column(
            "lastDeviceIp",
            sa.String(length=15),
            server_default="",
            nullable=True,
        ),
        schema="core_device",
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("DeviceInfo", "lastDeviceIp", schema="core_device")
    # ### end Alembic commands ###
